/*
 FioV3_wifiSingleClickControlWithSecurity
 
 this sketch is for FioV3 board with a RN wifi Xbee board, using password 173057F7A706AF9BBE65D51122A14CEE
 This sketch illustrates single click control for FioV3 with wifi board and using 128bit security
 
 (c)2013 Forward Computing and Control Pty. Ltd. 
 This code may be freely used for both private and commerical use.
 Provide this copyright is maintained.
 */

#include <pfodCHAP.h>
pfodCHAP parser;

// Pin 3 if the output being switched.
// give it a name:
int switchedOutput = 3;

// the setup routine runs once when you press reset:
void setup() {
  // Open serial communications and wait for port to open:
  Serial1.begin(9600);
  Serial.begin(9600);
  // allow a little time to connect the serialMonitor before running the rest of the setup.
  // also introduces a delay on power cycles to make it infeasable to deny access by using up all the power cycles.
  for (int i = 5; i>0; i--) {
    delay(1000);
    Serial.print(F(" "));
    Serial.print(i);
  }
  Serial.println();
  // call this BEFORE init so any debug output from init goes to the correct place
  parser.setDebugStream(&Serial); // default goes to parser.init serial connection, this optional call redirects the dubug output for FioV3 to the USB connection.
  // for FioV3 which has two serial connections Serial is the USB one used for debug output from the paser
  // There is NO debug output by default. You need to edit the C++ library code to turn on debug output, see the CPP code for details.
  // for UNO there is only one serial connection so 


  //  parser.init(&Serial1,0, F(""));  // start with no password to just test the connection, this will work with bluetooth connections also
  parser.init(&Serial1, 0, F("173057F7A706AF9BBE65D51122A14CEE")); // start with password given in GettingStarted docs
  // DO NOT use this password for real connections as it is well known. See the GettingStarted doc for a password generator

  parser.setIdleTimeout(0); // the default
  // this next setting is the idleTimeout default is 0mS i.e. never timeout
  // a non-zero setting is only used for wifi connections
  // setting this to non-zero value protects against a hacker taking over your connection and just hanging on to it
  // not sending any msgs but not releasing it, so preventing you from re-connecting.
  // when setting to non-zero you can use the pfod re-request time to ask the pfodApp to re-request a menu every so often to prevent the connection timing out
  // while the pfodApp is running. See the pfod Specification for details.

  // initialize the digital pin as an output.
  pinMode(switchedOutput, OUTPUT);
  digitalWrite(switchedOutput, LOW); // start with output low (off)  

  Serial.println("Initialization Finished:");

}

/*
 * !!! This disconnect( ) method is the one you need to implement for your particular hardware wifi board
 *
 * Need to be able to close the connection from the pfodDevice end
 * if an invalid hash is received or the idle timer times out
 *
 * This code forces the RN wifi Xbee module to drop the connection.
 *
 * @param io -- the stream that talks to the pfodApp (via the wifi)
 *  get this from parser.getPfodAppStream() 
 */
void disconnect(Stream *io) {
  // code what ever is needed to force the link to disconnect
  // typically get into cmd mode and either reboot or close the link
  // for RN wifi Xbee module need to wait >250mS send $$$ and then wait >250mS
  // then send close and exit 
  // should get EXIT back
  char EXIT[6] = { 'E', 'X', 'I', 'T', 13, 10 };
  boolean foundExit = false;
  while (!foundExit) {
    delay(300);
    io->write('$');
    io->write('$');
    io->write('$');
    delay(300);
    io->print("close\r");
    delay(300);
    io->print("exit\r");
    // wait for EXIT
    foundExit = io->find(EXIT, 6);
  }
}

// the loop routine runs over and over again forever:
void loop() {
  byte cmd = 0;
  cmd = parser.parse(); // do the parser stuff reading from Serial1 the wifi connection

  if (cmd != 0) { // have parsed a complete msg { to }
    if (cmd == pfodCHAP::DisconnectNow) {
      // need to disconnected
      // do disconnection stuff here and then call
      disconnect(parser.getPfodAppStream());
      parser.disconnected();
#ifdef DEBUG
      Serial.println(F("Disconnected"));
#endif
    } else if ('.' == cmd) {
      // pfodApp has connected and sent {.} it is asking for the main menu
      // toggle the output and send back Close Connection message with current state of output
      // either {!Output OFF}  or {!Output ON}
      parser.print(F("{!")); // send start of close connection message
      boolean outputState = digitalRead(switchedOutput);
      if (outputState == LOW) {
        digitalWrite(switchedOutput, HIGH); // was off turn on
        parser.print(F("Output is ON")); // change this to your own message
      } else {
        digitalWrite(switchedOutput, LOW); // was on turn off
        parser.print(F("Output is OFF")); // change this to your own message
      }
      parser.print(F("}")); // send closing } to complete pfod message,  pfodApp will exit and display your message
      // you can add any code here that is needed to close the connection on the micro side.
      disconnect(parser.getPfodAppStream());
      parser.disconnected(); // tell the parser we have disconnected and are waiting for a new connection
    }
    cmd = 0; // have processed this cmd now
    // so clear it and wait for next one
  }
}



