// Example of SingleClickControl for FioV3 Wifi
// Toggles D3 on and off with each connection
// display current state on mobile
// This example does not have any security
// See example under pfodCHAP for 128bit security

#include <pfodCmdParser.h> // include the library
pfodCmdParser parser; // create the cmd parser

// Pin 3 if the output being switched.
// give it a name:
int switchedOutput = 3;

// the setup routine runs once when you press reset:
void setup() {
  Serial.begin(9600);
  for (int i=3; i>0; i--) {
    // wait a few secs to see if we are being programmed
    delay(1000);
  }
  Serial1.begin(9600);
  // initialize the digital pin as an output.
  pinMode(switchedOutput, OUTPUT);
  digitalWrite(switchedOutput, LOW); // start with output low (off)  
}

// the loop routine runs over and over again forever:
void loop() {
  byte in = 0;
  byte cmd = 0;
  if (Serial1.available()) {
    in = Serial1.read(); // read the next char
    cmd = parser.parse(in); // pass it to the parser returns non-zero when a command is fully parsed
    if (cmd != 0) { // have parsed a complete msg { to }
      if ('.' == cmd) {
        // pfodApp has connected and sent {.} it is asking for the main menu
        // toggle the output and send back Close Connection message with current state of output
        // either {!Output OFF}  or {!Output ON}
        Serial1.print(F("{!")); // send start of close connection message
        boolean outputState = digitalRead(switchedOutput);
        if (outputState == LOW) {
          digitalWrite(switchedOutput, HIGH); // was off turn on
          Serial1.print(F("Output is ON")); // change this to your own message
        } else {
          digitalWrite(switchedOutput, LOW); // was on turn off
          Serial1.print(F("Output is OFF")); // change this to your own message
        }
        Serial1.print(F("}")); // send closing } to complete pfod message,  pfodApp will exit and display your message
        // you can add any code here that is needed to close the connection on the micro side.
        disconnect(&Serial1);
      }
    }
    cmd = 0; // have processed this cmd now
    // so clear it and wait for next one
  } // else no serial chars just loop
}

/*
 * !!! This disconnect( ) method is the one you need to implement for your particular hardware wifi board
 * If closes the connection on the Arduino side when pfodApp exits
 * This code forces the RN wifi Xbee module to drop the connection.
 *
 * @param io -- the stream that talks to the pfodApp (via the wifi)
 *  Serial1 in this case for FioV3 
 */
void disconnect(Stream *io) {
  // code what ever is needed to force the link to disconnect
  // typically get into cmd mode and either reboot or close the link
  // for RN wifi Xbee module need to wait >250mS send $$$ and then wait >250mS
  // then send close and exit 
  // should get EXIT back
  char EXIT[6] = { 'E', 'X', 'I', 'T', 13, 10 };
  boolean foundExit = false;
  while (!foundExit) {
    delay(300);
    io->write('$');
    io->write('$');
    io->write('$');
    delay(300);
    io->print("close\r");
    delay(300);
    io->print("exit\r");
    // wait for EXIT
    foundExit = io->find(EXIT, 6);
  }
}

